@@ -10,8 +10,10 @@
 #include <libetonyek/libetonyek.h>
 
 #include <cassert>
+#include <cstring>
 #include <memory>
 
+#include <boost/algorithm/string/predicate.hpp>
 #include <boost/optional.hpp>
 
 #include <libxml/xmlreader.h>
@@ -22,10 +24,13 @@
 #include "IWASnappyStream.h"
 #include "IWORKPresentationRedirector.h"
 #include "IWORKSpreadsheetRedirector.h"
+#include "IWORKSubDirStream.h"
 #include "IWORKTextRedirector.h"
 #include "IWORKTokenizer.h"
 #include "IWORKZlibStream.h"
+#include "KEY1Dictionary.h"
 #include "KEY1Parser.h"
+#include "KEY1Token.h"
 #include "KEY2Dictionary.h"
 #include "KEY2Parser.h"
 #include "KEY2Token.h"
@@ -117,9 +122,15 @@ bool probeXML(DetectionInfo &info)
   xmlTextReaderSetErrorHandler(reader.get(), handleError, nullptr);
 
   int ret = 0;
+  bool checkAPXL=false;
   do
   {
     ret = xmlTextReaderRead(reader.get());
+    if (ret==1 && XML_READER_TYPE_DOCUMENT_TYPE==xmlTextReaderNodeType(reader.get()))
+    {
+      auto name = xmlTextReaderConstName(reader.get());
+      if (name) checkAPXL=std::string((char const *)name)=="APXL";
+    }
   }
   while ((1 == ret) && (XML_READER_TYPE_ELEMENT != xmlTextReaderNodeType(reader.get())));
 
@@ -128,7 +139,6 @@ bool probeXML(DetectionInfo &info)
 
   const char *const name = char_cast(xmlTextReaderConstLocalName(reader.get()));
   const char *const ns = char_cast(xmlTextReaderConstNamespaceUri(reader.get()));
-
   if (probeXMLFormat(FORMAT_XML2, EtonyekDocument::TYPE_KEYNOTE, KEY2Token::NS_URI_KEY | KEY2Token::presentation,
                      KEY2Token::getTokenizer(), name, ns, info))
     return true;
@@ -138,7 +148,10 @@ bool probeXML(DetectionInfo &info)
   if (probeXMLFormat(FORMAT_XML2, EtonyekDocument::TYPE_PAGES, PAG1Token::NS_URI_SL | PAG1Token::document,
                      PAG1Token::getTokenizer(), name, ns, info))
     return true;
-
+  // Keynote 1 files define the document type with <!DOCTYPE APXL>
+  if (probeXMLFormat(FORMAT_XML1, EtonyekDocument::TYPE_KEYNOTE, KEY1Token::NS_URI_KEY | KEY1Token::presentation,
+                     KEY1Token::getTokenizer(), name, (ns||!checkAPXL) ? ns : "http://developer.apple.com/schemas/APXL", info))
+    return true;
   return false;
 }
 
@@ -150,23 +163,55 @@ bool probeBinary(DetectionInfo &info)
 
   EtonyekDocument::Type detected = EtonyekDocument::TYPE_UNKNOWN;
 
-  const IWAMessage header(info.m_input, headerLen);
+  const auto pos = uint64_t(info.m_input->tell());
+  const IWAMessage header(info.m_input, (unsigned long) headerLen);
 
   if (header.uint32(1) && header.message(2) && header.message(2).uint32(1) && (header.uint32(1).get() == 1))
   {
     switch (header.message(2).uint32(1).get())
     {
     case 1 :
-      // The app-specific object types for Keynote and Numbers overlap.
-      // So we use a structure check earlier to provisionally set the type as Keynote (or not).
-      // TODO: We likely need a better detection here: either parse the first
-      // object or find the document kind info somewhere in the document
-      // (Metadata/Properties.plist?).
-      detected = (info.m_type == EtonyekDocument::TYPE_KEYNOTE) ? EtonyekDocument::TYPE_KEYNOTE : EtonyekDocument::TYPE_NUMBERS;
+      if (header.message(2).uint32(3))
+      {
+        const auto dataLen = get(header.message(2).uint32(3));
+        const IWAMessage data(info.m_input, long(pos + headerLen), long(pos + headerLen + dataLen));
+        // keynote: presentation ref in 2
+        // number: sheet ref in 1
+        if (!data.message(1))
+          detected = EtonyekDocument::TYPE_KEYNOTE;
+        else if (!data.message(2))
+          detected = EtonyekDocument::TYPE_NUMBERS;
+        else
+        {
+          unsigned potentialRef[2];
+          for (unsigned test=1; test<=2; ++test)
+          {
+            auto ref=data.message(test).uint32(1).optional();
+            if (!ref)
+            {
+              detected = test==1 ? EtonyekDocument::TYPE_KEYNOTE : EtonyekDocument::TYPE_NUMBERS;
+              break;
+            }
+            potentialRef[test-1]=get(ref);
+          }
+          if (detected != EtonyekDocument::TYPE_UNKNOWN)
+            break;
+          // undecise, try to find the first ref
+          IWAObjectIndex objIndex(info.m_fragments, info.m_package);
+          objIndex.parse();
+          unsigned type;
+          boost::optional<IWAMessage> msg;
+          objIndex.queryObject(potentialRef[0], type, msg);
+          detected = msg && type==2 ?
+                     EtonyekDocument::TYPE_NUMBERS : EtonyekDocument::TYPE_KEYNOTE;
+        }
+      }
       break;
     case 10000 :
       detected = EtonyekDocument::TYPE_PAGES;
       break;
+    default:
+      break;
     }
   }
 
@@ -199,36 +244,90 @@ catch (...)
   return RVNGInputStreamPtr_t();
 }
 
+bool detectBinary(RVNGInputStreamPtr_t input, DetectionInfo &info)
+{
+  assert(input->isStructured());
+
+  if (input->existsSubStream("Metadata/DocumentIdentifier"))
+    info.m_package = input;
+
+  if (input->existsSubStream("Index.zip"))
+  {
+    RVNGInputStreamPtr_t zipInput = getSubStream(input, "Index.zip");
+    if (bool(zipInput))
+      input = zipInput;
+  }
+
+  const bool hasDocument = input->existsSubStream("Index/Document.iwa");
+
+  if (hasDocument)
+  {
+    info.m_format = FORMAT_BINARY;
+    info.m_fragments = input;
+    info.m_input = getUncompressedSubStream(input, "Index/Document.iwa", true);
+  }
+
+  return hasDocument;
+}
+
+RVNGInputStreamPtr_t queryTopDirStream(const RVNGInputStreamPtr_t &input)
+{
+  assert(input->isStructured());
+
+  string top;
+
+  // find the common top level dir of all substream names, if there is one
+  for (unsigned i = 0; i < input->subStreamCount(); ++i)
+  {
+    const char *const path = input->subStreamName(i);
+    if (path)
+    {
+      if (top.empty())
+      {
+        // initialize top dir
+        const char *const pos = std::strchr(path, '/');
+        if (pos)
+          top.assign(path, pos - path);
+        else
+          top = path;
+      }
+      else
+      {
+        // check that the current path starts with top dir
+        if (!boost::starts_with(path, top))
+          return RVNGInputStreamPtr_t();
+        const char end = path[top.size()];
+        if (end != '/' && end != '\0')
+          return RVNGInputStreamPtr_t();
+      }
+    }
+  }
+
+  RVNGInputStreamPtr_t stream;
+  if (!top.empty())
+    stream.reset(new IWORKSubDirStream(input, top));
+
+  return stream;
+}
+
 bool detect(const RVNGInputStreamPtr_t &input, DetectionInfo &info)
 {
   if (input->isStructured())
   {
-    info.m_package = input;
-
     if ((info.m_format == FORMAT_BINARY) || (info.m_format == FORMAT_UNKNOWN))
     {
-      RVNGInputStreamPtr_t binaryInput(input);
-      const bool isPackage(binaryInput->existsSubStream("Metadata/DocumentIdentifier"));
-      if (binaryInput->existsSubStream("Index.zip"))
+      if (!detectBinary(input, info))
       {
-        RVNGInputStreamPtr_t zipInput = getSubStream(binaryInput, "Index.zip");
-        if (bool(zipInput))
-          binaryInput = zipInput;
+        RVNGInputStreamPtr_t dir = queryTopDirStream(input);
+        if (dir)
+          detectBinary(dir, info);
       }
-      info.m_fragments = binaryInput;
-      if (binaryInput->existsSubStream("Index/Document.iwa"))
-      {
-        if (!isPackage)
-          info.m_package.reset();
-        info.m_format = FORMAT_BINARY;
-        info.m_input = getUncompressedSubStream(binaryInput, "Index/Document.iwa", true);
-      }
-      if ((info.m_type == EtonyekDocument::TYPE_UNKNOWN) && binaryInput->existsSubStream("Index/MasterSlide.iwa"))
-        info.m_type = EtonyekDocument::TYPE_KEYNOTE;
     }
 
     if ((info.m_format == FORMAT_XML2) || (info.m_format == FORMAT_UNKNOWN))
     {
+      info.m_package = input;
+
       if ((info.m_type == EtonyekDocument::TYPE_KEYNOTE) || (info.m_type == EtonyekDocument::TYPE_UNKNOWN))
       {
         if (input->existsSubStream("index.apxl"))
@@ -260,8 +359,10 @@ bool detect(const RVNGInputStreamPtr_t &input, DetectionInfo &info)
       }
     }
 
-    if ((info.m_format == FORMAT_XML1) || (info.m_format == FORMAT_UNKNOWN))
+    if (info.m_format == FORMAT_XML1 || info.m_format == FORMAT_UNKNOWN)
     {
+      info.m_package = input;
+
       if (input->existsSubStream("presentation.apxl"))
       {
         info.m_type = EtonyekDocument::TYPE_KEYNOTE;
@@ -280,7 +381,7 @@ bool detect(const RVNGInputStreamPtr_t &input, DetectionInfo &info)
   {
     try
     {
-      info.m_input.reset(new IWORKZlibStream(input));
+      info.m_input = std::make_shared<IWORKZlibStream>(input);
     }
     catch (...)
     {
@@ -318,25 +419,6 @@ bool detect(const RVNGInputStreamPtr_t &input, DetectionInfo &info)
 
 }
 
-namespace
-{
-
-shared_ptr<IWORKParser> makeKeynoteParser(const unsigned version, const RVNGInputStreamPtr_t &input, const RVNGInputStreamPtr_t &package, KEYCollector &collector, KEY2Dictionary &dict)
-{
-  shared_ptr<IWORKParser> parser;
-
-  if (1 == version)
-    parser.reset(new KEY1Parser(input, package, collector));
-  else if ((2 <= version) && (5 >= version))
-    parser.reset(new KEY2Parser(input, package, collector, dict));
-  else
-    assert(0);
-
-  return parser;
-}
-
-}
-
 ETONYEKAPI EtonyekDocument::Confidence EtonyekDocument::isSupported(librevenge::RVNGInputStream *const input, EtonyekDocument::Type *type) try
 {
   if (!input)
@@ -375,11 +457,18 @@ ETONYEKAPI bool EtonyekDocument::parse(librevenge::RVNGInputStream *const input,
 
   IWORKPresentationRedirector redirector(generator);
   KEYCollector collector(&redirector);
-  if (info.m_format == FORMAT_XML2)
+  if (info.m_format == FORMAT_XML1)
+  {
+    KEY1Dictionary dict;
+    input->seek(0, librevenge::RVNG_SEEK_SET);
+    shared_ptr<KEY1Parser> key1Parser(new KEY1Parser(info.m_input, info.m_package, collector, dict));
+    return key1Parser->parse();
+  }
+  else if (info.m_format == FORMAT_XML2)
   {
     KEY2Dictionary dict;
-    const shared_ptr<IWORKParser> parser = makeKeynoteParser(info.m_format, info.m_input, info.m_package, collector, dict);
-    return parser->parse();
+    shared_ptr<KEY2Parser> key2Parser(new KEY2Parser(info.m_input, info.m_package, collector, dict));
+    return key2Parser->parse();
   }
   else if (info.m_format == FORMAT_BINARY)
   {
