@@ -34,12 +34,23 @@ using std::vector;
 
 struct Coord
 {
+  Coord()
+    : m_coord(0)
+    , m_absolute(false)
+  {
+  }
   unsigned m_coord;
   bool m_absolute;
 };
 
 struct Address
 {
+  Address()
+    : m_column()
+    , m_row()
+    , m_table()
+  {
+  }
   optional<Coord> m_column;
   optional<Coord> m_row;
   optional<string> m_table;
@@ -47,22 +58,43 @@ struct Address
 
 typedef std::pair<Address, Address> AddressRange;
 
+struct TrueOrFalseFunc
+{
+  TrueOrFalseFunc()
+    : m_name()
+  {
+  }
+  string m_name;
+};
+
 struct Function;
 struct PrefixOp;
 struct InfixOp;
 struct PostfixOp;
 struct PExpr;
 
-typedef variant<double, string, Address, AddressRange, recursive_wrapper<PrefixOp>, recursive_wrapper<InfixOp>, recursive_wrapper<PostfixOp>, recursive_wrapper<Function>, recursive_wrapper<PExpr> > Expression;
+typedef variant<double, string, TrueOrFalseFunc, Address, AddressRange, recursive_wrapper<PrefixOp>, recursive_wrapper<InfixOp>, recursive_wrapper<PostfixOp>, recursive_wrapper<Function>, recursive_wrapper<PExpr> > Expression;
 
 struct PrefixOp
 {
+  PrefixOp()
+    : m_op(' ')
+    , m_expr()
+  {
+  }
   char m_op;
   Expression m_expr;
 };
 
 struct InfixOp
 {
+  InfixOp()
+    : m_op()
+    , m_left()
+    , m_right()
+  {
+  }
+
   string m_op;
   Expression m_left;
   Expression m_right;
@@ -70,18 +102,32 @@ struct InfixOp
 
 struct PostfixOp
 {
+  PostfixOp()
+    : m_op(' ')
+    , m_expr()
+  {
+  }
   char m_op;
   Expression m_expr;
 };
 
 struct Function
 {
+  Function()
+    : m_name()
+    , m_args()
+  {
+  }
   string m_name;
   vector<Expression> m_args;
 };
 
 struct PExpr
 {
+  PExpr()
+    : m_expr()
+  {
+  }
   Expression m_expr;
 };
 
@@ -93,6 +139,11 @@ BOOST_FUSION_ADAPT_STRUCT(
   (unsigned, m_coord)
 )
 
+BOOST_FUSION_ADAPT_STRUCT(
+  libetonyek::TrueOrFalseFunc,
+  (std::string, m_name)
+)
+
 BOOST_FUSION_ADAPT_STRUCT(
   libetonyek::Address,
   (optional<std::string>, m_table)
@@ -146,16 +197,46 @@ unsigned parseRowName(const vector<char> &columnName)
   unsigned columnNumber = 0;
 
   for (char it : columnName)
-    columnNumber = 26 * columnNumber + (toupper(it) - 'A') + 1;
+    columnNumber = 26 * columnNumber + unsigned(toupper(it) - 'A') + 1;
 
   return columnNumber;
 }
 
+#if defined _MSC_VER
+#pragma warning(push)
+#pragma warning(disable : 4100) // unreferenced formal parameter in boost
+#endif
+
 template<typename Iterator>
 struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
 {
   FormulaGrammar()
     : FormulaGrammar::base_type(formula, "formula")
+    , function()
+    , trueOrFalseFunction()
+    , expression()
+    , formula()
+    , term()
+    , pExpr()
+    , address()
+    , addressSpecialColumn()
+    , addressSpecialRow()
+    , range()
+    , columnName()
+    , column()
+    , row()
+    , number()
+    , str()
+    , table()
+    , infixLit()
+    , trueOrFalseFunctionLit()
+    , prefixOp()
+    , infixOp()
+    , postfixOp()
+    , prefixLit()
+    , postfixLit()
+    , rangeSpecial()
+    , mappedName()
   {
     using ascii::char_;
     using ascii::string;
@@ -176,6 +257,7 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
 
     number %= double_;
     str %= lit('\"') >> +(char_ - '\"') >> '\"';
+    trueOrFalseFunctionLit %= string("TRUE")|string("FALSE");
     prefixLit %= char_('+') | char_('-');
     infixLit %=
       char_('+') | char_('-') | char_('*') | char_('/')
@@ -225,29 +307,27 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
 
     prefixOp %= prefixLit >> term;
     infixOp %= term >> infixLit >> expression;
+    trueOrFalseFunction %= trueOrFalseFunctionLit;
     postfixOp %= term >> postfixLit;
 
-    function %= (no_case[mappedName] | +alnum) >> '(' >> -(argument % ',') >> ')';
-
-    argument %=
-      range
-      | '(' >> range >> ')'
-      | expression
-      ;
+    function %= (no_case[mappedName] | +alnum) >> '(' >> -(expression % ',') >> ')';
 
     pExpr %= '(' >> expression >> ')';
 
     term %=
       str
-      | number
       | function
+      | trueOrFalseFunction
+      | range
       | address
+      | number
       | prefixOp
       | pExpr
       ;
 
     expression %=
-      infixOp
+      lit(' ') >> expression
+      | infixOp
       | postfixOp
       | term
       ;
@@ -256,6 +336,7 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
 
     number.name("number");
     str.name("str");
+    trueOrFalseFunction.name("trueOrFalseFunction");
     prefixOp.name("prefixOp");
     infixOp.name("infixOp");
     row.name("row");
@@ -269,7 +350,6 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
     term.name("term");
     formula.name("formula");
     pExpr.name("pExpr");
-    argument.name("argument");
     rangeSpecial.name("rangeSpecial");
   }
 
@@ -278,14 +358,15 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
   }
 
   qi::rule<Iterator, Function()> function;
-  qi::rule<Iterator, Expression()> expression, formula, term, argument;
+  qi::rule<Iterator, TrueOrFalseFunc()> trueOrFalseFunction;
+  qi::rule<Iterator, Expression()> expression, formula, term;
   qi::rule<Iterator, PExpr()> pExpr;
   qi::rule<Iterator, Address()> address, addressSpecialColumn, addressSpecialRow;
   qi::rule<Iterator, AddressRange()> range;
   qi::rule<Iterator, unsigned()> columnName;
   qi::rule<Iterator, Coord()> column, row;
   qi::rule<Iterator, double()> number;
-  qi::rule<Iterator, string()> str, table, infixLit;
+  qi::rule<Iterator, string()> str, table, infixLit, trueOrFalseFunctionLit;
   qi::rule<Iterator, PrefixOp()> prefixOp;
   qi::rule<Iterator, InfixOp()> infixOp;
   qi::rule<Iterator, PostfixOp()> postfixOp;
@@ -295,6 +376,10 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
   qi::symbols<char, string> mappedName;
 };
 
+#if defined _MSC_VER
+#pragma warning(pop)
+#endif
+
 }
 
 namespace
@@ -304,8 +389,10 @@ using std::ostringstream;
 
 struct Printer : public boost::static_visitor<void>
 {
-  Printer(ostringstream &out)
+  Printer(ostringstream &out, int offsetColumn, int offsetRow)
     : m_out(out)
+    , m_offsetColumn(offsetColumn)
+    , m_offsetRow(offsetRow)
   {
   }
 
@@ -335,33 +422,38 @@ struct Printer : public boost::static_visitor<void>
     m_out << ']';
   }
 
+  void operator()(const TrueOrFalseFunc &val) const
+  {
+    m_out << val.m_name << "()";
+  }
+
   void operator()(const recursive_wrapper<PrefixOp> &val) const
   {
     m_out << val.get().m_op;
-    apply_visitor(Printer(m_out), val.get().m_expr);
+    apply_visitor(Printer(m_out, m_offsetColumn, m_offsetRow), val.get().m_expr);
   }
 
   void operator()(const recursive_wrapper<InfixOp> &val) const
   {
-    apply_visitor(Printer(m_out), val.get().m_left);
+    apply_visitor(Printer(m_out, m_offsetColumn, m_offsetRow), val.get().m_left);
     m_out << val.get().m_op;
-    apply_visitor(Printer(m_out), val.get().m_right);
+    apply_visitor(Printer(m_out, m_offsetColumn, m_offsetRow), val.get().m_right);
   }
 
   void operator()(const recursive_wrapper<PostfixOp> &val) const
   {
-    apply_visitor(Printer(m_out), val.get().m_expr);
+    apply_visitor(Printer(m_out, m_offsetColumn, m_offsetRow), val.get().m_expr);
     m_out << val.get().m_op;
   }
 
   void operator()(const recursive_wrapper<Function> &val) const
   {
     m_out << val.get().m_name << '(';
-    for (vector<Expression>::const_iterator it = val.get().m_args.begin(); it != val.get().m_args.end(); ++it)
+    for (auto it = val.get().m_args.begin(); it != val.get().m_args.end(); ++it)
     {
       if (it != val.get().m_args.begin())
         m_out << ';';
-      apply_visitor(Printer(m_out), *it);
+      apply_visitor(Printer(m_out, m_offsetColumn, m_offsetRow), *it);
     }
     m_out << ')';
   }
@@ -369,7 +461,7 @@ struct Printer : public boost::static_visitor<void>
   void operator()(const recursive_wrapper<PExpr> &val) const
   {
     m_out << '(';
-    apply_visitor(Printer(m_out), val.get().m_expr);
+    apply_visitor(Printer(m_out, m_offsetColumn, m_offsetRow), val.get().m_expr);
     m_out << ')';
   }
 
@@ -381,31 +473,40 @@ private:
     m_out  << '.';
     if (val.m_column)
     {
-      if (get(val.m_column).m_absolute)
-        m_out << '$';
-
-      unsigned column = get(val.m_column).m_coord;
-      vector<char> columnNumerals;
-      columnNumerals.reserve(4);
-      while (column != 0)
+      int offsetColumn=get(val.m_column).m_absolute ? 0 : m_offsetColumn;
+      if (int(get(val.m_column).m_coord)+offsetColumn>=0)
       {
-        if (column > 0)
-          --column;
-        columnNumerals.push_back('A' + column % 26);
-        column /= 26;
+        if (get(val.m_column).m_absolute)
+          m_out << '$';
+
+        auto column = unsigned(int(get(val.m_column).m_coord)+offsetColumn);
+        vector<char> columnNumerals;
+        columnNumerals.reserve(4);
+        while (column != 0)
+        {
+          if (column > 0)
+            --column;
+          columnNumerals.push_back(char('A' + column % 26));
+          column /= 26;
+        }
+        copy(columnNumerals.rbegin(), columnNumerals.rend(), std::ostream_iterator<char>(m_out));
       }
-      copy(columnNumerals.rbegin(), columnNumerals.rend(), std::ostream_iterator<char>(m_out));
     }
     if (val.m_row)
     {
-      if (get(val.m_row).m_absolute)
-        m_out << '$';
-      m_out << get(val.m_row).m_coord;
+      int offsetRow=get(val.m_row).m_absolute ? 0 : m_offsetRow;
+      if (int(get(val.m_row).m_coord)+offsetRow>=0)
+      {
+        if (get(val.m_row).m_absolute)
+          m_out << '$';
+        m_out << int(get(val.m_row).m_coord)+offsetRow;
+      }
     }
   }
 
 private:
   ostringstream &m_out;
+  int m_offsetColumn, m_offsetRow;
 };
 
 }
@@ -416,9 +517,12 @@ namespace
 struct Collector : public boost::static_visitor<>
 {
 
-  explicit Collector(librevenge::RVNGPropertyListVector &propsVector, const IWORKTableNameMapPtr_t &tableNameMap)
+  explicit Collector(librevenge::RVNGPropertyListVector &propsVector, const IWORKTableNameMapPtr_t &tableNameMap,
+                     int offsetColumn, int offsetRow)
     : m_propsVector(propsVector)
     , m_tableNameMap(tableNameMap)
+    , m_offsetColumn(offsetColumn)
+    , m_offsetRow(offsetRow)
   { }
 
   void operator()(double val) const
@@ -437,6 +541,20 @@ struct Collector : public boost::static_visitor<>
     m_propsVector.append(props);
   }
 
+  void operator()(const TrueOrFalseFunc &val) const
+  {
+    librevenge::RVNGPropertyList props1;
+    props1.insert("librevenge:type", "librevenge-function");
+    props1.insert("librevenge:function", val.m_name.c_str());
+    m_propsVector.append(props1);
+    librevenge::RVNGPropertyList props2;
+    props2.insert("librevenge:type", "librevenge-operator");
+    props2.insert("librevenge:operator", "(");
+    m_propsVector.append(props2);
+    props2.insert("librevenge:operator", ")");
+    m_propsVector.append(props2);
+  }
+
   void operator()(const Address &val) const
   {
 
@@ -452,16 +570,24 @@ struct Collector : public boost::static_visitor<>
         props.insert("librevenge:sheet-name", get(val.m_table).c_str());
     }
 
-    if (val.m_column && get(val.m_column).m_coord>0)
+    if (val.m_column)
     {
-      props.insert("librevenge:column-absolute", get(val.m_column).m_absolute);
-      props.insert("librevenge:column", int(get(val.m_column).m_coord-1));
+      int offsetColumn=get(val.m_column).m_absolute ? 0 : m_offsetColumn;
+      if (int(get(val.m_column).m_coord)+offsetColumn>0)
+      {
+        props.insert("librevenge:column-absolute", get(val.m_column).m_absolute);
+        props.insert("librevenge:column", int(get(val.m_column).m_coord)-1+offsetColumn);
+      }
     }
 
-    if (val.m_row && get(val.m_row).m_coord>0)
+    if (val.m_row)
     {
-      props.insert("librevenge:row-absolute", get(val.m_row).m_absolute);
-      props.insert("librevenge:row", int(get(val.m_row).m_coord-1));
+      int offsetRow=get(val.m_row).m_absolute ? 0 : m_offsetRow;
+      if (int(get(val.m_row).m_coord)+offsetRow>0)
+      {
+        props.insert("librevenge:row-absolute", get(val.m_row).m_absolute);
+        props.insert("librevenge:row", int(get(val.m_row).m_coord)-1+offsetRow);
+      }
     }
 
     m_propsVector.append(props);
@@ -472,25 +598,41 @@ struct Collector : public boost::static_visitor<>
     librevenge::RVNGPropertyList props;
     props.insert("librevenge:type", "librevenge-cells");
 
-    if (val.first.m_column && get(val.first.m_column).m_coord>0)
+    if (val.first.m_column)
     {
-      props.insert("librevenge:start-column-absolute", get(val.first.m_column).m_absolute);
-      props.insert("librevenge:start-column", int(get(val.first.m_column).m_coord-1));
+      int offsetColumn=get(val.first.m_column).m_absolute ? 0 : m_offsetColumn;
+      if (int(get(val.first.m_column).m_coord)+offsetColumn>0)
+      {
+        props.insert("librevenge:start-column-absolute", get(val.first.m_column).m_absolute);
+        props.insert("librevenge:start-column", int(get(val.first.m_column).m_coord)-1+offsetColumn);
+      }
     }
-    if (val.first.m_row && get(val.first.m_row).m_coord>0)
+    if (val.first.m_row)
     {
-      props.insert("librevenge:start-row-absolute", get(val.first.m_row).m_absolute);
-      props.insert("librevenge:start-row", int(get(val.first.m_row).m_coord-1));
+      int offsetRow=get(val.first.m_row).m_absolute ? 0 : m_offsetRow;
+      if (int(get(val.first.m_row).m_coord)+offsetRow>0)
+      {
+        props.insert("librevenge:start-row-absolute", get(val.first.m_row).m_absolute);
+        props.insert("librevenge:start-row", int(get(val.first.m_row).m_coord)-1+offsetRow);
+      }
     }
-    if (val.second.m_column && get(val.second.m_column).m_coord>0)
+    if (val.second.m_column)
     {
-      props.insert("librevenge:end-column-absolute", get(val.second.m_column).m_absolute);
-      props.insert("librevenge:end-column", int(get(val.second.m_column).m_coord-1));
+      int offsetColumn=get(val.second.m_column).m_absolute ? 0 : m_offsetColumn;
+      if (int(get(val.second.m_column).m_coord)+offsetColumn>0)
+      {
+        props.insert("librevenge:end-column-absolute", get(val.second.m_column).m_absolute);
+        props.insert("librevenge:end-column", int(get(val.second.m_column).m_coord)-1+offsetColumn);
+      }
     }
-    if (val.second.m_row && get(val.second.m_row).m_coord>0)
+    if (val.second.m_row)
     {
-      props.insert("librevenge:end-row-absolute", get(val.second.m_row).m_absolute);
-      props.insert("librevenge:end-row", int(get(val.second.m_row).m_coord-1));
+      int offsetRow=get(val.second.m_row).m_absolute ? 0 : m_offsetRow;
+      if (int(get(val.second.m_row).m_coord)+offsetRow>0)
+      {
+        props.insert("librevenge:end-row-absolute", get(val.second.m_row).m_absolute);
+        props.insert("librevenge:end-row", int(get(val.second.m_row).m_coord)-1+offsetRow);
+      }
     }
     m_propsVector.append(props);
   }
@@ -501,25 +643,27 @@ struct Collector : public boost::static_visitor<>
     props.insert("librevenge:type", "librevenge-operator");
     props.insert("librevenge:operator", val.get().m_op);
     m_propsVector.append(props);
-    apply_visitor(Collector(m_propsVector, m_tableNameMap), val.get().m_expr);
+    apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), val.get().m_expr);
   }
 
   void operator()(const recursive_wrapper<InfixOp> &val) const
   {
-    apply_visitor(Collector(m_propsVector, m_tableNameMap), val.get().m_left);
+    apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), val.get().m_left);
     librevenge::RVNGPropertyList props;
     props.insert("librevenge:type", "librevenge-operator");
     props.insert("librevenge:operator", val.get().m_op.c_str());
     m_propsVector.append(props);
-    apply_visitor(Collector(m_propsVector, m_tableNameMap), val.get().m_right);
+    apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), val.get().m_right);
   }
 
   void operator()(const recursive_wrapper<PostfixOp> &val) const
   {
-    apply_visitor(Collector(m_propsVector, m_tableNameMap), val.get().m_expr);
+    apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), val.get().m_expr);
     librevenge::RVNGPropertyList props;
     props.insert("librevenge:type", "librevenge-operator");
-    props.insert("librevenge:operator", val.get().m_op);
+    std::string op;
+    op+=val.get().m_op;
+    props.insert("librevenge:operator", op.c_str());
     m_propsVector.append(props);
   }
 
@@ -539,11 +683,11 @@ struct Collector : public boost::static_visitor<>
     props.insert("librevenge:type", "librevenge-operator");
     props.insert("librevenge:operator", ";");
 
-    for (vector<Expression>::const_iterator it = val.get().m_args.begin(); it != val.get().m_args.end(); ++it)
+    for (auto it = val.get().m_args.begin(); it != val.get().m_args.end(); ++it)
     {
       if (it != val.get().m_args.begin())
         m_propsVector.append(props);
-      apply_visitor(Collector(m_propsVector, m_tableNameMap), *it);
+      apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), *it);
     }
     librevenge::RVNGPropertyList props3;
     props3.insert("librevenge:type", "librevenge-operator");
@@ -558,7 +702,7 @@ struct Collector : public boost::static_visitor<>
     props.insert("librevenge:operator", "(");
     m_propsVector.append(props);
 
-    apply_visitor(Collector(m_propsVector, m_tableNameMap), val.get().m_expr);
+    apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), val.get().m_expr);
 
     librevenge::RVNGPropertyList props1;
     props1.insert("librevenge:type", "librevenge-operator");
@@ -570,17 +714,23 @@ struct Collector : public boost::static_visitor<>
 private:
   librevenge::RVNGPropertyListVector &m_propsVector;
   const IWORKTableNameMapPtr_t &m_tableNameMap;
+  int m_offsetColumn, m_offsetRow;
 };
 
 }
 
 struct IWORKFormula::Impl
 {
+  Impl() :
+    m_formula()
+  {
+  }
   Expression m_formula;
 };
 
-IWORKFormula::IWORKFormula()
+IWORKFormula::IWORKFormula(const boost::optional<unsigned> &hc)
   : m_impl(new Impl())
+  , m_hc(hc)
 {
 }
 
@@ -590,20 +740,49 @@ bool IWORKFormula::parse(const std::string &formula)
   string::const_iterator it = formula.begin();
   string::const_iterator end = formula.end();
   const bool r = qi::phrase_parse(it, end, grammar, ascii::space, m_impl->m_formula);
-  return r && (it == end);
+  if (!r || it!=end)
+  {
+    ETONYEK_DEBUG_MSG(("IWORKFormula::parse: can not parse %s\n", formula.c_str()));
+    return false;
+  }
+  return true;
 }
 
-const std::string IWORKFormula::str() const
+const std::string IWORKFormula::str(const boost::optional<unsigned> &hc) const
 {
   ostringstream out;
   out << '=';
-  apply_visitor(Printer(out), m_impl->m_formula);
+  int offsetCol=0, offsetRow=0;
+  if (!computeOffset(hc, offsetCol, offsetRow))
+    offsetCol=offsetRow=0;
+  apply_visitor(Printer(out, offsetCol, offsetRow), m_impl->m_formula);
   return out.str();
 }
 
-void IWORKFormula::write(librevenge::RVNGPropertyListVector &formula, const IWORKTableNameMapPtr_t &tableNameMap) const
+void IWORKFormula::write(const boost::optional<unsigned> &hc, librevenge::RVNGPropertyListVector &formula, const IWORKTableNameMapPtr_t &tableNameMap) const
 {
-  apply_visitor(Collector(formula, tableNameMap), m_impl->m_formula);
+  int offsetCol=0, offsetRow=0;
+  if (!computeOffset(hc, offsetCol, offsetRow))
+    offsetCol=offsetRow=0;
+  apply_visitor(Collector(formula, tableNameMap, offsetCol, offsetRow), m_impl->m_formula);
+}
+
+bool IWORKFormula::computeOffset(const boost::optional<unsigned> &hc, int &offsetColumn, int &offsetRow) const
+{
+  offsetColumn=offsetRow=0;
+  if (!m_hc && !hc)
+    return true;
+  if (!m_hc || !hc)
+  {
+    ETONYEK_DEBUG_MSG(("IWORKFormula::parse: called without cell positions\n"));
+    return false;
+  }
+  if (get(m_hc)==get(hc)) return true;
+  int prevRow=(int) get(m_hc)/256, prevColumn=(int) get(m_hc)%256;
+  int row=(int) get(hc)/256, column=(int) get(hc)%256;
+  offsetColumn=column-prevColumn;
+  offsetRow=row-prevRow;
+  return true;
 }
 
 } // namespace libetonyek
