@@ -7,6 +7,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include "libetonyek_xml.h"
+
 #include "IWORKXMLContext.h"
 
 namespace libetonyek
@@ -16,6 +18,11 @@ IWORKXMLContext::~IWORKXMLContext()
 {
 }
 
+void IWORKXMLContext::CDATA(const char */*value*/)
+{
+  ETONYEK_DEBUG_MSG(("IWORKXMLContext::cData: find unexpected CDATA block\n"));
+}
+
 }
 
 /* vim:set shiftwidth=2 softtabstop=2 expandtab: */
