@@ -10,8 +10,11 @@
 #ifndef KEYCOLLECTOR_H_INCLUDED
 #define KEYCOLLECTOR_H_INCLUDED
 
+#include <deque>
+
 #include "IWORKCollector.h"
 #include "IWORKPath_fwd.h"
+#include "IWORKStyle_fwd.h"
 #include "KEYTypes.h"
 #include "KEYTypes_fwd.h"
 
@@ -22,7 +25,7 @@ class KEYCollector : public IWORKCollector
 {
 public:
   explicit KEYCollector(IWORKDocumentInterface *document);
-  ~KEYCollector();
+  ~KEYCollector() override;
 
   // collector functions
 
@@ -30,7 +33,7 @@ public:
 
   KEYLayerPtr_t collectLayer();
   void insertLayer(const KEYLayerPtr_t &layer);
-  void collectPage();
+  KEYSlidePtr_t collectSlide();
 
   KEYPlaceholderPtr_t collectTextPlaceholder(const IWORKStylePtr_t &style, bool title);
   void insertTextPlaceholder(const KEYPlaceholderPtr_t &placeholder);
@@ -39,6 +42,14 @@ public:
 
   void collectStickyNote();
 
+  void setSlideStyle(const IWORKStylePtr_t &style);
+
+  // helper functions
+
+  void startDocument();
+  void sendSlides(const std::deque<KEYSlidePtr_t> &slides);
+  void endDocument();
+
   void startSlides();
   void endSlides();
   void startThemes();
@@ -49,21 +60,30 @@ public:
   void startLayer();
   void endLayer();
 
+protected:
+  bool m_inSlides;
+
 private:
-  void drawNotes();
-  void drawStickyNotes();
+  void insertSlide(const KEYSlidePtr_t &slide, bool isMaster, const boost::optional<std::string> &pageName=boost::none);
+  void drawTable() override;
+  void drawMedia(double x, double y, const librevenge::RVNGPropertyList &data) override;
+  void fillShapeProperties(librevenge::RVNGPropertyList &props) override;
+  bool createFrameStylesForTextBox() const override
+  {
+    return false;
+  }
+  void drawTextBox(const IWORKTextPtr_t &text, const glm::dmat3 &trafo, const IWORKGeometryPtr_t &boundingBox, const librevenge::RVNGPropertyList &style) override;
 
 private:
   IWORKSize m_size;
 
+  KEYSlidePtr_t m_currentSlide;
   IWORKOutputElements m_notes;
-  KEYStickyNotes_t m_stickyNotes;
+  IWORKOutputElements m_stickyNotes;
 
   bool m_pageOpened;
   bool m_layerOpened;
   int m_layerCount;
-
-  bool m_paint;
 };
 
 } // namespace libetonyek
