@@ -117,7 +117,7 @@ uint64_t readUVar(const RVNGInputStreamPtr_t &input)
   while (!input->isEnd() && cont)
   {
     const unsigned char c = readU8(input);
-    bytes.push_back(c & ~0x80);
+    bytes.push_back((unsigned char)(c & ~0x80));
     cont = c & 0x80;
   }
 
@@ -230,12 +230,9 @@ double pt2in(const double d)
 double deg2rad(double value)
 {
   // normalize range
-  while (360 <= value)
-    value -= 360;
-  while (0 > value)
+  value = std::fmod(value, 360);
+  if (value < 0)
     value += 360;
-
-  // convert
   return etonyek_pi / 180 * value;
 }
 
@@ -255,9 +252,9 @@ librevenge::RVNGString makeColor(const IWORKColor &color)
 {
   // TODO: alpha
 
-  const unsigned r = color.m_red * 256 - 0.5;
-  const unsigned g = color.m_green * 256 - 0.5;
-  const unsigned b = color.m_blue * 256 - 0.5;
+  const auto r = unsigned(color.m_red * 256 - 0.5);
+  const auto g = unsigned(color.m_green * 256 - 0.5);
+  const auto b = unsigned(color.m_blue * 256 - 0.5);
 
   librevenge::RVNGString str;
   str.sprintf("#%.2x%.2x%.2x", r, g, b);
@@ -273,12 +270,12 @@ librevenge::RVNGString makeColor(const IWORKGradient &gradient)
     return "#ffffff";
   }
   const double middle=(gradient.m_stops.front().m_fraction+gradient.m_stops.back().m_fraction)/2;
-  const unsigned red = (middle*gradient.m_stops.front().m_color.m_red+
-                        (1-middle)*gradient.m_stops.back().m_color.m_red)* 256 - 0.5;
-  const unsigned green = (middle*gradient.m_stops.front().m_color.m_green+
-                          (1-middle)*gradient.m_stops.back().m_color.m_green)* 256 - 0.5;
-  const unsigned blue = (middle*gradient.m_stops.front().m_color.m_blue+
-                         (1-middle)*gradient.m_stops.back().m_color.m_blue)* 256 - 0.5;
+  const auto red = unsigned((middle*gradient.m_stops.front().m_color.m_red+
+                             (1-middle)*gradient.m_stops.back().m_color.m_red)* 256 - 0.5);
+  const auto green = unsigned((middle*gradient.m_stops.front().m_color.m_green+
+                               (1-middle)*gradient.m_stops.back().m_color.m_green)* 256 - 0.5);
+  const auto blue = unsigned((middle*gradient.m_stops.front().m_color.m_blue+
+                              (1-middle)*gradient.m_stops.back().m_color.m_blue)* 256 - 0.5);
 
   librevenge::RVNGString str;
   str.sprintf("#%.2x%.2x%.2x", red, green, blue);
@@ -288,7 +285,7 @@ librevenge::RVNGString makeColor(const IWORKGradient &gradient)
 
 void writeBorder(const IWORKStroke &stroke, const char *const name, librevenge::RVNGPropertyList &props)
 {
-  if (stroke.m_type == IWORK_STROKE_TYPE_NONE)
+  if (stroke.m_pattern.m_type == IWORK_STROKE_TYPE_NONE)
     return;
 
   librevenge::RVNGString border;
@@ -297,12 +294,12 @@ void writeBorder(const IWORKStroke &stroke, const char *const name, librevenge::
 
   // The format can represent arbitrary patterns, but we have to
   // fit them to the limited number of options ODF allows...
-  if ((stroke.m_type == IWORK_STROKE_TYPE_DASHED) && (stroke.m_pattern.size() >= 2))
+  if ((stroke.m_pattern.m_type == IWORK_STROKE_TYPE_DASHED) && (stroke.m_pattern.m_values.size() >= 2))
   {
-    const double x = stroke.m_pattern[0];
-    const double y = stroke.m_pattern[1];
+    const double x = stroke.m_pattern.m_values[0];
+    const double y = stroke.m_pattern.m_values[1];
     // checkme: maybe we can use <sf:stroke ... sf:cap|butt ...>
-    if (((x / y) < 0.01) || ((y / x) < 0.01)) // arbitrarily picked constant
+    if (x != 0 && y != 0 && (((x / y) < 0.01) || ((y / x) < 0.01))) // arbitrarily picked constant
       border.append(" dotted");
     else
       border.append(" dashed");
